@@ -3,6 +3,7 @@ import re
 from openshift.dynamic.exceptions import ConflictError
 
 from ocp_resources.constants import TIMEOUT_4MINUTES
+from ocp_resources.logger import get_logger
 from ocp_resources.node import Node
 from ocp_resources.node_network_configuration_enactment import (
     NodeNetworkConfigurationEnactment,
@@ -12,6 +13,9 @@ from ocp_resources.resource import Resource, ResourceEditor
 from ocp_resources.utils import TimeoutExpiredError, TimeoutSampler
 
 
+LOGGER = get_logger(name=__name__)
+
+
 class NNCPConfigurationFailed(Exception):
     pass
 
@@ -102,6 +106,7 @@ class NodeNetworkConfigurationPolicy(Resource):
         self.set_ipv6 = set_ipv6
         self.success_timeout = success_timeout
         self.max_unavailable = max_unavailable
+        self.res = None
         self.ipv4_ports_backup_dict = {}
         self.ipv6_ports_backup_dict = {}
         self.nodes = self._nodes()
@@ -247,6 +252,9 @@ class NodeNetworkConfigurationPolicy(Resource):
         return self.res
 
     def _get_port_from_nns(self, port_name):
+        if not self.nodes:
+            return None
+
         nns = NodeNetworkState(name=self.nodes[0].name)
         _port = [_iface for _iface in nns.interfaces if _iface["name"] == port_name]
         return _port[0] if _port else None
@@ -292,20 +300,20 @@ class NodeNetworkConfigurationPolicy(Resource):
             func=self.update,
             resource_dict=resource,
         )
-        self.logger.info(f"Applying {resource}")
+        LOGGER.info(f"Applying {resource}")
         for _ in samples:
             return
 
-    def deploy(self, wait=False):
+    def deploy(self):
         self.ipv4_ports_backup()
         self.ipv6_ports_backup()
 
-        self.create(wait=wait)
+        self.create(body=self.res)
         try:
             self.wait_for_status_success()
             return self
         except Exception as exp:
-            self.logger.error(exp)
+            LOGGER.error(exp)
             super().__exit__(exception_type=None, exception_value=None, traceback=None)
             raise
 
@@ -315,7 +323,7 @@ class NodeNetworkConfigurationPolicy(Resource):
                 self._absent_interface()
                 self.wait_for_status_success()
             except Exception as exp:
-                self.logger.error(exp)
+                LOGGER.error(exp)
 
         super().clean_up()
 
@@ -337,7 +345,6 @@ class NodeNetworkConfigurationPolicy(Resource):
             }
         ).update()
 
-    @property
     def status(self):
         for condition in self.instance.status.conditions:
             if condition["type"] == self.Conditions.Type.AVAILABLE:
@@ -385,12 +392,12 @@ class NodeNetworkConfigurationPolicy(Resource):
         self.wait_for_configuration_conditions_unknown_or_progressing()
 
         samples = TimeoutSampler(
-            wait_timeout=self.success_timeout, sleep=1, func=lambda: self.status
+            wait_timeout=self.success_timeout, sleep=1, func=self.status
         )
         try:
             for sample in samples:
                 if sample == self.Conditions.Reason.SUCCESSFULLY_CONFIGURED:
-                    self.logger.info(f"NNCP {self.name} configured Successfully")
+                    LOGGER.info(f"NNCP {self.name} configured Successfully")
                     return sample
 
                 elif sample == no_match_node_condition_reason:
@@ -404,7 +411,7 @@ class NodeNetworkConfigurationPolicy(Resource):
                     )
 
         except (TimeoutExpiredError, NNCPConfigurationFailed):
-            self.logger.error(
+            LOGGER.error(
                 f"Unable to configure NNCP {self.name} "
                 f"{f'nodes: {[node.name for node in self.nodes]}' if self.nodes else ''}"
             )
@@ -449,7 +456,7 @@ class NodeNetworkConfigurationPolicy(Resource):
             try:
                 nnce.wait_for_conditions()
             except TimeoutExpiredError:
-                self.logger.error(f"Failed to get NNCE {nnce.name} status")
+                LOGGER.error(f"Failed to get NNCE {nnce.name} status")
                 continue
 
             for nnce_cond in nnce.instance.status.conditions:
