@@ -7,42 +7,49 @@ from io import StringIO
 from signal import SIGINT, signal
 
 import kubernetes
+import urllib3
 import yaml
 from openshift.dynamic import DynamicClient
-from openshift.dynamic.exceptions import ConflictError, NotFoundError
+from openshift.dynamic.exceptions import (
+    ConflictError,
+    InternalServerError,
+    NotFoundError,
+    ServerTimeoutError,
+)
 from openshift.dynamic.resource import ResourceField
 from packaging.version import Version
 
 from ocp_resources.constants import (
-    DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
     NOT_FOUND_ERROR_EXCEPTION_DICT,
     PROTOCOL_ERROR_EXCEPTION_DICT,
-    TIMEOUT_1MINUTE,
     TIMEOUT_4MINUTES,
 )
 from ocp_resources.logger import get_logger
-from ocp_resources.utils import (
-    TimeoutExpiredError,
-    TimeoutSampler,
-    skip_existing_resource_creation_teardown,
-)
+from ocp_resources.utils import TimeoutExpiredError, TimeoutSampler
 
 
-LOGGER = get_logger(__name__)
+DEFAULT_CLUSTER_RETRY_EXCEPTIONS = {
+    ConnectionAbortedError: [],
+    ConnectionResetError: [],
+    InternalServerError: [
+        "etcdserver: leader changed",
+        "etcdserver: request timed out",
+    ],
+    ServerTimeoutError: [],
+}
+
+LOGGER = get_logger(name=__name__)
 MAX_SUPPORTED_API_VERSION = "v2"
 
 
-def kube_v1_api(api_client):
-    return kubernetes.client.CoreV1Api(api_client=api_client)
-
-
 def _collect_instance_data(directory, resource_object):
     with open(os.path.join(directory, f"{resource_object.name}.yaml"), "w") as fd:
         fd.write(resource_object.instance.to_str())
 
 
 def _collect_pod_logs(dyn_client, resource_item, **kwargs):
-    return kube_v1_api(api_client=dyn_client.client).read_namespaced_pod_log(
+    kube_v1_api = kubernetes.client.CoreV1Api(api_client=dyn_client.client)
+    return kube_v1_api.read_namespaced_pod_log(
         name=resource_item.metadata.name,
         namespace=resource_item.metadata.namespace,
         **kwargs,
@@ -132,25 +139,6 @@ def _get_api_version(dyn_client, api_group, kind):
     return res.group_version
 
 
-def _get_client(config_file=None, context=None):
-    """
-    Get a kubernetes client.
-
-    Args:
-        config_file (str): path to a kubeconfig file.
-        context (str): name of the context to use.
-
-    Returns:
-        DynamicClient: a kubernetes client.
-    """
-    return DynamicClient(
-        client=kubernetes.config.new_client_from_config(
-            config_file=config_file,
-            context=context,
-        )
-    )
-
-
 def sub_resource_level(current_class, owner_class, parent_class):
     # return the name of the last class in MRO list that is not one of base
     # classes; otherwise return None
@@ -246,7 +234,6 @@ class Resource:
     api_group = None
     api_version = None
     singular_name = None
-    timeout_seconds = TIMEOUT_1MINUTE
 
     class Status:
         SUCCEEDED = "Succeeded"
@@ -257,7 +244,6 @@ class Resource:
         COMPLETED = "Completed"
         RUNNING = "Running"
         TERMINATING = "Terminating"
-        ERROR = "Error"
 
     class Condition:
         UPGRADEABLE = "Upgradeable"
@@ -299,7 +285,6 @@ class Resource:
         CONFIG_OPENSHIFT_IO = "config.openshift.io"
         CONSOLE_OPENSHIFT_IO = "console.openshift.io"
         EVENTS_K8S_IO = "events.k8s.io"
-        FLAVOR_KUBEVIRT_IO = "flavor.kubevirt.io"
         FORKLIFT_KONVEYOR_IO = "forklift.konveyor.io"
         HCO_KUBEVIRT_IO = "hco.kubevirt.io"
         HOSTPATHPROVISIONER_KUBEVIRT_IO = "hostpathprovisioner.kubevirt.io"
@@ -314,7 +299,6 @@ class Resource:
         MACHINE_OPENSHIFT_IO = "machine.openshift.io"
         MACHINECONFIGURATION_OPENSHIFT_IO = "machineconfiguration.openshift.io"
         MAISTRA_IO = "maistra.io"
-        MIGRATIONS_KUBEVIRT_IO = "migrations.kubevirt.io"
         MONITORING_COREOS_COM = "monitoring.coreos.com"
         NETWORKADDONSOPERATOR_NETWORK_KUBEVIRT_IO = (
             "networkaddonsoperator.network.kubevirt.io"
@@ -330,7 +314,6 @@ class Resource:
         OS_TEMPLATE_KUBEVIRT_IO = "os.template.kubevirt.io"
         PACKAGES_OPERATORS_COREOS_COM = "packages.operators.coreos.com"
         POLICY = "policy"
-        POOL_KUBEVIRT_IO = "pool.kubevirt.io"
         PROJECT_OPENSHIFT_IO = "project.openshift.io"
         RBAC_AUTHORIZATION_K8S_IO = "rbac.authorization.k8s.io"
         RIPSAW_CLOUDBULLDOZER_IO = "ripsaw.cloudbulldozer.io"
@@ -344,12 +327,12 @@ class Resource:
         SSP_KUBEVIRT_IO = "ssp.kubevirt.io"
         STORAGE_K8S_IO = "storage.k8s.io"
         STORAGECLASS_KUBERNETES_IO = "storageclass.kubernetes.io"
-        SUBRESOURCES_KUBEVIRT_IO = "subresources.kubevirt.io"
         TEMPLATE_KUBEVIRT_IO = "template.kubevirt.io"
         TEMPLATE_OPENSHIFT_IO = "template.openshift.io"
         UPLOAD_CDI_KUBEVIRT_IO = "upload.cdi.kubevirt.io"
         V2V_KUBEVIRT_IO = "v2v.kubevirt.io"
         VM_KUBEVIRT_IO = "vm.kubevirt.io"
+        MIGRATIONS_KUBEVIRT_IO = "migrations.kubevirt.io"
 
     class ApiVersion:
         V1 = "v1"
@@ -369,27 +352,12 @@ class Resource:
         dry_run=None,
         node_selector=None,
         node_selector_labels=None,
-        config_file=None,
-        context=None,
-        timeout_seconds=TIMEOUT_1MINUTE,
     ):
         """
-        Create an API resource
+        Create a API resource
 
         Args:
             name (str): Resource name
-            client (DynamicClient): Dynamic client for connecting to a remote cluster
-            teardown (bool): Indicates if this resource would need to be deleted
-            privileged_client (DynamicClient): Instance of Dynamic client
-            yaml_file (str): yaml file for the resource
-            delete_timeout (int): timeout associated with delete action
-            dry_run (bool): dry run
-            node_selector (str): node selector
-            node_selector_labels (str): node selector labels
-            config_file (str): Path to config file for connecting to remote cluster.
-            context (str): Context name for connecting to remote cluster.
-            timeout_seconds (int): timeout for a get api call, call out be terminated after this many seconds
-
         """
         if not self.api_group and not self.api_version:
             raise NotImplementedError(
@@ -401,11 +369,27 @@ class Resource:
         self.privileged_client = privileged_client
         self.yaml_file = yaml_file
         self.resource_dict = None  # Filled in case yaml_file is not None
-        self.config_file = config_file
-        self.context = context
         if not (self.name or self.yaml_file):
             raise ValueError("name or yaml file is required")
 
+        if not self.client:
+            try:
+                self.client = DynamicClient(
+                    client=kubernetes.config.new_client_from_config()
+                )
+            except (
+                kubernetes.config.ConfigException,
+                urllib3.exceptions.MaxRetryError,
+            ):
+                LOGGER.error(
+                    "You need to be logged into a cluster or have $KUBECONFIG env configured"
+                )
+                raise
+        if not self.api_version:
+            self.api_version = _get_api_version(
+                dyn_client=self.client, api_group=self.api_group, kind=self.kind
+            )
+
         self.teardown = teardown
         self.timeout = timeout
         self.delete_timeout = delete_timeout
@@ -413,10 +397,6 @@ class Resource:
         self.node_selector = node_selector
         self.node_selector_labels = node_selector_labels
         self.node_selector_spec = self._prepare_node_selector_spec()
-        self.res = None
-        self.yaml_file_contents = None
-        self.logger = get_logger(name=f"{__name__.rsplit('.')[0]} {self.kind}")
-        self.timeout_seconds = timeout_seconds
 
     def _prepare_node_selector_spec(self):
         if self.node_selector:
@@ -436,25 +416,22 @@ class Resource:
             dict: Resource dict.
         """
         if self.yaml_file:
-            if not self.yaml_file_contents:
-                if isinstance(self.yaml_file, StringIO):
-                    self.yaml_file_contents = self.yaml_file.read()
-                else:
-                    with open(self.yaml_file, "r") as stream:
-                        self.yaml_file_contents = stream.read()
+            if isinstance(self.yaml_file, StringIO):
+                data = self.yaml_file.read()
+            else:
+                with open(self.yaml_file, "r") as stream:
+                    data = stream.read()
 
-            self.resource_dict = yaml.safe_load(stream=self.yaml_file_contents)
+            self.resource_dict = yaml.safe_load(stream=data)
             self.resource_dict.get("metadata", {}).pop("resourceVersion", None)
             self.name = self.resource_dict["metadata"]["name"]
-        else:
-            self._set_client_and_api_version()
-            self.resource_dict = {
-                "apiVersion": self.api_version,
-                "kind": self.kind,
-                "metadata": {"name": self.name},
-            }
+            return self.resource_dict
 
-        return self.resource_dict
+        return {
+            "apiVersion": self.api_version,
+            "kind": self.kind,
+            "metadata": {"name": self.name},
+        }
 
     def to_dict(self):
         """
@@ -467,6 +444,16 @@ class Resource:
         return self.deploy()
 
     def __exit__(self, exception_type, exception_value, traceback):
+        # For debug, export kind (class name) + NOTEARDOWN to skip resource teardown.
+        # For example:export VirtualMachineInstanceNOTEARDOWN=True
+        skip_teardown_env = f"{self.kind}NOTEARDOWN"
+        no_teardown_from_environment = os.environ.get(skip_teardown_env)
+        if no_teardown_from_environment:
+            LOGGER.warning(
+                f"Skip teardown. Got {skip_teardown_env}={no_teardown_from_environment}"
+            )
+            return
+
         if self.teardown:
             self.clean_up()
 
@@ -474,38 +461,8 @@ class Resource:
         self.__exit__(exception_type=None, exception_value=None, traceback=None)
         sys.exit(signal_received)
 
-    def deploy(self, wait=False):
-        """
-        For debug, export REUSE_IF_RESOURCE_EXISTS to skip resource create.
-        Spaces are important in the export dict
-
-        Examples:
-            To skip creation of all resources by kind:
-                export REUSE_IF_RESOURCE_EXISTS="{Pod: {}}"
-
-            To skip creation of resource by name (on all namespaces or non-namespaced resources):
-                export REUSE_IF_RESOURCE_EXISTS="{Pod: {<pod-name>:}}"
-
-            To skip creation of resource by name and namespace:
-                export REUSE_IF_RESOURCE_EXISTS="{Pod: {<pod-name>: <pod-namespace>}}"
-
-            To skip creation of multiple resources:
-                export REUSE_IF_RESOURCE_EXISTS="{Namespace: {<namespace-name>:}, Pod: {<pod-name>: <pod-namespace>}}"
-        """
-        _resource = None
-        _export_str = "REUSE_IF_RESOURCE_EXISTS"
-        skip_resource_kind_create_if_exists = os.environ.get(_export_str)
-        if skip_resource_kind_create_if_exists:
-            _resource = skip_existing_resource_creation_teardown(
-                resource=self,
-                export_str=_export_str,
-                user_exported_args=skip_resource_kind_create_if_exists,
-            )
-
-            if _resource:
-                return _resource
-
-        self.create(wait=wait)
+    def deploy(self):
+        self.create()
         return self
 
     def clean_up(self):
@@ -517,36 +474,6 @@ class Resource:
                     f"Log collector failed to collect info for {self.kind} {self.name}\nexception: {exception_}"
                 )
 
-        """
-        For debug, export SKIP_RESOURCE_TEARDOWN to skip resource teardown.
-        Spaces are important in the export dict
-
-        Examples:
-            To skip teardown of all resources by kind:
-                export SKIP_RESOURCE_TEARDOWN="{Pod: {}}"
-
-            To skip teardown of resource by name (on all namespaces):
-                export SKIP_RESOURCE_TEARDOWN="{Pod: {<pod-name>:}}"
-
-            To skip teardown of resource by name and namespace:
-                export SKIP_RESOURCE_TEARDOWN="{Pod: {<pod-name>: <pod-namespace>}}"
-
-            To skip teardown of multiple resources:
-                export SKIP_RESOURCE_TEARDOWN="{Namespace: {<namespace-name>:}, Pod: {<pod-name>: <pod-namespace>}}"
-        """
-        _export_str = "SKIP_RESOURCE_TEARDOWN"
-        skip_resource_teardown = os.environ.get(_export_str)
-        if skip_resource_teardown and skip_existing_resource_creation_teardown(
-            resource=self,
-            export_str=_export_str,
-            user_exported_args=skip_resource_teardown,
-            check_exists=False,
-        ):
-            self.logger.warning(
-                f"Skip resource {self.kind} {self.name} teardown. Got {_export_str}={skip_resource_teardown}"
-            )
-            return
-
         self.delete(wait=True, timeout=self.delete_timeout)
 
     @classmethod
@@ -558,10 +485,8 @@ class Resource:
 
         get_kwargs = {"singular_name": singular_name} if singular_name else {}
         return dyn_client.resources.get(
-            kind=cls.kind,
-            api_version=cls.api_version,
-            **get_kwargs,
-        ).get(*args, **kwargs, timeout_seconds=cls.timeout_seconds)
+            kind=cls.kind, api_version=cls.api_version, **get_kwargs
+        ).get(*args, **kwargs)
 
     def _prepare_singular_name_kwargs(self, **kwargs):
         kwargs = kwargs if kwargs else {}
@@ -570,17 +495,6 @@ class Resource:
 
         return kwargs
 
-    def _set_client_and_api_version(self):
-        if not self.client:
-            self.client = _get_client(
-                config_file=self.config_file, context=self.context
-            )
-
-        if not self.api_version:
-            self.api_version = _get_api_version(
-                dyn_client=self.client, api_group=self.api_group, kind=self.kind
-            )
-
     def full_api(self, **kwargs):
         """
         Get resource API
@@ -600,8 +514,6 @@ class Resource:
         Returns:
             Resource: Resource object.
         """
-        self._set_client_and_api_version()
-
         kwargs = self._prepare_singular_name_kwargs(**kwargs)
 
         return self.client.resources.get(
@@ -623,14 +535,13 @@ class Resource:
         Raises:
             TimeoutExpiredError: If resource not exists.
         """
-        self.logger.info(f"Wait until {self.kind} {self.name} is created")
+        LOGGER.info(f"Wait until {self.kind} {self.name} is created")
         samples = TimeoutSampler(
             wait_timeout=timeout,
             sleep=sleep,
             exceptions_dict={
                 **PROTOCOL_ERROR_EXCEPTION_DICT,
                 **NOT_FOUND_ERROR_EXCEPTION_DICT,
-                **DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
             },
             func=lambda: self.exists,
         )
@@ -648,7 +559,7 @@ class Resource:
         Raises:
             TimeoutExpiredError: If resource still exists.
         """
-        self.logger.info(f"Wait until {self.kind} {self.name} is deleted")
+        LOGGER.info(f"Wait until {self.kind} {self.name} is deleted")
         return self.client_wait_deleted(timeout=timeout)
 
     @property
@@ -693,14 +604,11 @@ class Resource:
             TimeoutExpiredError: If resource in not in desire status.
         """
         stop_status = stop_status if stop_status else self.Status.FAILED
-        self.logger.info(f"Wait for {self.kind} {self.name} status to be {status}")
+        LOGGER.info(f"Wait for {self.kind} {self.name} status to be {status}")
         samples = TimeoutSampler(
             wait_timeout=timeout,
             sleep=sleep,
-            exceptions_dict={
-                **PROTOCOL_ERROR_EXCEPTION_DICT,
-                **DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
-            },
+            exceptions_dict=PROTOCOL_ERROR_EXCEPTION_DICT,
             func=self.api.get,
             field_selector=f"metadata.name=={self.name}",
             namespace=self.namespace,
@@ -722,16 +630,15 @@ class Resource:
 
         except TimeoutExpiredError:
             if current_status:
-                self.logger.error(
-                    f"Status of {self.kind} {self.name} is {current_status}"
-                )
+                LOGGER.error(f"Status of {self.kind} {self.name} is {current_status}")
             raise
 
-    def create(self, wait=False):
+    def create(self, body=None, wait=False):
         """
         Create resource.
 
         Args:
+            body (dict): Resource data to create.
             wait (bool) : True to wait for resource status.
 
         Returns:
@@ -741,20 +648,33 @@ class Resource:
             ValueMismatch: When body value doesn't match class value
         """
         data = self.to_dict()
-        self.logger.info(f"Create {self.kind} {self.name}")
-        self.logger.info(f"Posting {data}")
-        self.logger.debug(f"\n{yaml.dump(data)}")
+        if body:
+            kind = body["kind"]
+            name = body.get("name")
+            api_version = body["apiVersion"]
+            if kind != self.kind:
+                raise ValueMismatch(f"{kind} != {self.kind}")
+            if name and name != self.name:
+                raise ValueMismatch(f"{name} != {self.name}")
+            if api_version != self.api_version:
+                raise ValueMismatch(f"{api_version} != {self.api_version}")
+
+            data.update(body)
+
+        LOGGER.info(f"Create {self.kind} {self.name}")
+        LOGGER.info(f"Posting {data}")
+        LOGGER.debug(f"\n{yaml.dump(data)}")
         res = self.api.create(body=data, namespace=self.namespace, dry_run=self.dry_run)
         if wait and res:
             return self.wait()
         return res
 
     def delete(self, wait=False, timeout=TIMEOUT_4MINUTES, body=None):
-        self.logger.info(f"Delete {self.kind} {self.name}")
+        LOGGER.info(f"Delete {self.kind} {self.name}")
         if self.exists:
             data = self.instance.to_dict()
-            self.logger.info(f"Deleting {data}")
-            self.logger.debug(f"\n{yaml.dump(data)}")
+            LOGGER.info(f"Deleting {data}")
+            LOGGER.debug(f"\n{yaml.dump(data)}")
 
         try:
             res = self.api.delete(name=self.name, namespace=self.namespace, body=body)
@@ -775,7 +695,7 @@ class Resource:
         Returns:
            str: Status
         """
-        self.logger.info(f"Get {self.kind} {self.name} status")
+        LOGGER.info(f"Get {self.kind} {self.name} status")
         return self.instance.status.phase
 
     def update(self, resource_dict):
@@ -785,8 +705,8 @@ class Resource:
         Args:
             resource_dict: Resource dictionary
         """
-        self.logger.info(f"Update {self.kind} {self.name}:\n{resource_dict}")
-        self.logger.debug(f"\n{yaml.dump(resource_dict)}")
+        LOGGER.info(f"Update {self.kind} {self.name}:\n{resource_dict}")
+        LOGGER.debug(f"\n{yaml.dump(resource_dict)}")
         self.api.patch(
             body=resource_dict,
             namespace=self.namespace,
@@ -798,14 +718,15 @@ class Resource:
         Replace resource metadata.
         Use this to remove existing field. (update() will only update existing fields)
         """
-        self.logger.info(f"Replace {self.kind} {self.name}: \n{resource_dict}")
-        self.logger.debug(f"\n{yaml.dump(resource_dict)}")
+        LOGGER.info(f"Replace {self.kind} {self.name}: \n{resource_dict}")
+        LOGGER.debug(f"\n{yaml.dump(resource_dict)}")
         self.api.replace(body=resource_dict, name=self.name, namespace=self.namespace)
 
     @staticmethod
     def retry_cluster_exceptions(
         func, exceptions_dict=DEFAULT_CLUSTER_RETRY_EXCEPTIONS, **kwargs
     ):
+
         sampler = TimeoutSampler(
             wait_timeout=10,
             sleep=1,
@@ -818,29 +739,17 @@ class Resource:
             return sample
 
     @classmethod
-    def get(
-        cls,
-        dyn_client=None,
-        config_file=None,
-        context=None,
-        singular_name=None,
-        *args,
-        **kwargs,
-    ):
+    def get(cls, dyn_client, singular_name=None, *args, **kwargs):
         """
         Get resources
 
         Args:
-            dyn_client (DynamicClient): Open connection to remote cluster.
-            config_file (str): Path to config file for connecting to remote cluster.
-            context (str): Context name for connecting to remote cluster.
-            singular_name (str): Resource kind (in lowercase), in use where we have multiple matches for resource.
+            dyn_client (DynamicClient): Open connection to remote cluster
+            singular_name (str): Resource kind (in lowercase), in use where we have multiple matches for resource
 
         Returns:
             generator: Generator of Resources of cls.kind
         """
-        if not dyn_client:
-            dyn_client = _get_client(config_file=config_file, context=context)
 
         def _get():
             _resources = cls._prepare_resources(
@@ -890,7 +799,7 @@ class Resource:
         Raises:
             TimeoutExpiredError: If Pod condition in not in desire status.
         """
-        self.logger.info(
+        LOGGER.info(
             f"Wait for {self.kind}/{self.name}'s '{condition}' condition to be '{status}'"
         )
         samples = TimeoutSampler(
@@ -980,23 +889,12 @@ class NamespacedResource(Resource):
             raise ValueError("name and namespace or yaml file is required")
 
     @classmethod
-    def get(
-        cls,
-        dyn_client=None,
-        config_file=None,
-        context=None,
-        singular_name=None,
-        raw=False,
-        *args,
-        **kwargs,
-    ):
+    def get(cls, dyn_client, singular_name=None, raw=False, *args, **kwargs):
         """
         Get resources
 
         Args:
             dyn_client (DynamicClient): Open connection to remote cluster
-            config_file (str): Path to config file for connecting to remote cluster.
-            context (str): Context name for connecting to remote cluster.
             singular_name (str): Resource kind (in lowercase), in use where we have multiple matches for resource
             raw (bool): If True return raw object from openshift-restclient-python
 
@@ -1004,9 +902,6 @@ class NamespacedResource(Resource):
         Returns:
             generator: Generator of Resources of cls.kind
         """
-        if not dyn_client:
-            dyn_client = _get_client(config_file=config_file, context=context)
-
         _resources = cls._prepare_resources(
             dyn_client=dyn_client, singular_name=singular_name, *args, **kwargs
         )
